@@ -35,4 +35,8 @@ DataStore* guppi_datastore();
 
 void guppi_replace_datastore(DataStore* ds);
 
+#ifdef GNOME_ENABLE_DEBUG
+void guppi_create_debug_datasets();
+#endif
+
 #endif
