@@ -412,6 +412,7 @@ guppi_axis_update (GnomeCanvasItem *item, double *update_affine, ArtSVP *clip_pa
 
           while (n_texts < gint(ticks.size()))
             {
+              // This is so unbelievably broken (for example, doesn't show-hide right)
               GnomeCanvasItem* txt = 
                 gnome_canvas_item_new(GNOME_CANVAS_GROUP(ga->item.parent),
                                       guppi_canvas_text_get_type(),
@@ -585,6 +586,29 @@ guppi_axis_update (GnomeCanvasItem *item, double *update_affine, ArtSVP *clip_pa
             }
           g_assert(textlist == NULL);
         }
+
+      if (item->object.flags & GNOME_CANVAS_ITEM_VISIBLE)
+        {
+          /* Show all the text items */
+          GSList* tmp = ga->texts;
+          while (tmp != NULL)
+            {
+              gnome_canvas_item_show(GNOME_CANVAS_ITEM(tmp->data));
+              
+              tmp = g_slist_next(tmp);
+            }
+        }
+      else 
+        {
+          /* Hide all the text items */
+          GSList* tmp = ga->texts;
+          while (tmp != NULL)
+            {
+              gnome_canvas_item_hide(GNOME_CANVAS_ITEM(tmp->data));
+              
+              tmp = g_slist_next(tmp);
+            }
+        }
     } 
   else 
     {
