@@ -62,6 +62,11 @@ public:
   DateData* cast_to_date();
   LabelData* cast_to_label();
 
+  const ScalarData* cast_to_scalar() const;     
+  const CategoricalData* cast_to_categorical() const;
+  const DateData* cast_to_date() const;
+  const LabelData* cast_to_label() const;
+
   Data();
   virtual ~Data();
                    
