@@ -82,6 +82,7 @@ public:
 
 private:
   GFont* default_font_;
+  bool already_tried_default_font_;
 };
 
 static GnomeFrontend gnomefe;
@@ -121,6 +122,8 @@ guppi_gnome_shutdown_library()
       mpe = 0;
     }
 
+  guppi_shutdown_fonts();
+
   guppi_shutdown_library();
 }
 
@@ -211,6 +214,22 @@ guppi_install_menuitem_hint(GtkWidget* menuitem, GnomeAppBar* bar,
                       bar);
 }
 
+
+GtkWidget* 
+guppi_make_labelled_widget(const gchar* text, GtkWidget* child)
+{
+  GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
+  GtkWidget* label = gtk_label_new(text);
+
+  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+
+  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
+  gtk_box_pack_end(GTK_BOX(hbox), child, TRUE, TRUE, 0);
+
+  return hbox;
+}
+
+
 gchar* 
 guppi_logo_file()
 {
@@ -310,6 +329,8 @@ onrealize_seticon(GtkWindow* window, gpointer data)
   GdkWindow* icon = icon_window_new();
   if (icon != 0)
     {
+      g_debug("Setting icon for window %p", GTK_WIDGET(window)->window);
+
       gdk_window_set_icon(GTK_WIDGET(window)->window, icon, NULL, NULL);
 
       gdk_window_unref(icon);
@@ -337,7 +358,8 @@ guppi_setup_window(GtkWindow* window, const gchar* wmname)
 /////////////////// GnomeFrontend
 
 GnomeFrontend::GnomeFrontend()
-  : default_font_(0)
+  : default_font_(0),
+    already_tried_default_font_(false)
 {
 
 
@@ -346,10 +368,45 @@ GnomeFrontend::GnomeFrontend()
 VFont* 
 GnomeFrontend::default_font()
 {
-  if (default_font_ == 0)
+  if (default_font_ == 0 && !already_tried_default_font_)
     {
-      default_font_ = new GFont;
-      default_font_->ref(); // default font is never freed
+      default_font_ = GFont::get_font("Times", GFont::Normal, false, 18);
+
+      if (default_font_ == 0)
+        default_font_ = GFont::get_font("Helvetica", GFont::Normal, false, 18);
+
+      if (default_font_ == 0)
+        default_font_ =  GFont::get_font("fixed", GFont::Normal, false, 18);
+
+      if (default_font_ == 0)
+        {
+          // pick first font in list
+          GFontList* fl = guppi_get_font_list();
+          
+          g_assert(fl != 0);
+          
+          GFontList::iterator i = fl->begin();
+
+          if (i != fl->end())
+            {
+              default_font_ = *i;
+            }
+        }
+
+      if (default_font_ != 0)
+        {
+          default_font_->ref(); // default font is never freed
+
+          g_debug("Default font: %s", default_font_->font_name());
+        }
+      else
+        {
+          gnome_error_dialog(_("Couldn't get a default font from the GNOME print library.\n"
+                               "This indicates a misconfigured GNOME print that can't find any fonts.\n"
+                               "Guppi will use an X font instead, but it will be slow, and layout and printing may not work properly."));
+        }
+
+      already_tried_default_font_ = true;
     }
   
   return default_font_;
