@@ -273,13 +273,45 @@ parallel_shell_sort(XType* xsorted, YType* ysorted, int N)
 #endif
 }
 
+// For small sections that qsort doesn't like, this is simpler 
+//  and faster than the shell sort
+template <class XType, class YType>
+static inline void
+parallel_insertion_sort(XType* xsorted, YType* ysorted, int N)
+{
+  // not done yet
+
+#ifdef GNOME_ENABLE_DEBUG_SLOW
+  if (N > 1)
+    {
+      guint q = 1;
+      XType last_x = xsorted[0];
+      while (q < N)
+        {
+          if (last_x > xsorted[q])
+            g_warning("Insertion sort: %u out of order!", q);
+
+          last_x = xsorted[q];
+          ++q;
+        }
+    }
+#endif
+}
+
 template <class XType, class YType>
 static void parallel_sort(XType* xsorted, YType* ysorted, unsigned int N)
 {
-  if (N == 1) return;
+  if (N < 2) 
+    return;
 
   // FIXME figure out the max stack size for real.
-  unsigned int* stck = new unsigned int[N+7]; // far too large but definitely big enough.
+  // MAX_UINT on 64-bit is 2^64; log-base-2 of 2^64 is 64; we need two ints per 
+  //  stack entry; thus the stack must be 128 ints; double it for 128-bit-safety 
+  //  and add a few for luck :-)
+  // If you are better at math than me and think this is wrong, please fix. :-)
+#define PSORT_STACK_SIZE 260
+#define PSORT_STACK_SIZE_MINUS_FOUR 256
+  unsigned int stck[PSORT_STACK_SIZE];
   unsigned int top = 2;
   
   // The stack contains pairs of indices denoting ranges to sort
@@ -425,6 +457,7 @@ static void parallel_sort(XType* xsorted, YType* ysorted, unsigned int N)
               }
 #endif
 
+              g_assert(top < PSORT_STACK_SIZE_MINUS_FOUR);
 
               // if (strt < lo) // already known, since we have progress.
 
@@ -432,61 +465,70 @@ static void parallel_sort(XType* xsorted, YType* ysorted, unsigned int N)
               stck[top+1] = lo; // pivot is the limit
               top += 2;
               
+              ++lo; // one past pivot
               if (lo < stop)
                 {
-                  stck[top] = lo + 1; // one past pivot
+                  stck[top] = lo;
                   stck[top+1] = stop;
                   top += 2;
                 }
             }
         }
-      else if (diff == 3)
+      else
         {
-          const unsigned int pivot = strt + 1;
-
-          --stop;
-
-          if (xsorted[pivot] < xsorted[strt])
+          switch (diff)
             {
-              psort_swap(&xsorted[strt], &xsorted[pivot]);
-              psort_swap(&ysorted[strt], &ysorted[pivot]);
-            }
+
+            case 3:
+              {
+                const unsigned int pivot = strt + 1;
+
+                --stop;
+
+                if (xsorted[pivot] < xsorted[strt])
+                  {
+                    psort_swap(&xsorted[strt], &xsorted[pivot]);
+                    psort_swap(&ysorted[strt], &ysorted[pivot]);
+                  }
       
-          if (xsorted[stop] < xsorted[pivot])
-            {
-              psort_swap(&xsorted[stop], &xsorted[pivot]);
-              psort_swap(&ysorted[stop], &ysorted[pivot]);
+                if (xsorted[stop] < xsorted[pivot])
+                  {
+                    psort_swap(&xsorted[stop], &xsorted[pivot]);
+                    psort_swap(&ysorted[stop], &ysorted[pivot]);
           
-              if (xsorted[pivot] < xsorted[strt])
-                {
-                  psort_swap(&xsorted[strt], &xsorted[pivot]);
-                  psort_swap(&ysorted[strt], &ysorted[pivot]);
-                }
+                    if (xsorted[pivot] < xsorted[strt])
+                      {
+                        psort_swap(&xsorted[strt], &xsorted[pivot]);
+                        psort_swap(&ysorted[strt], &ysorted[pivot]);
+                      }
+                  }
+
+                //          g_assert(xsorted[strt] <= xsorted[pivot]);
+                //          g_assert(xsorted[pivot] <= xsorted[stop]);
+
+              }
+              break;
+
+            case 2:
+              {
+                --stop;
+                if (xsorted[strt] > xsorted[stop])
+                  {
+                    psort_swap(&xsorted[strt], &xsorted[stop]);
+                    psort_swap(&ysorted[strt], &ysorted[stop]);
+                  }
+              }
+              break;
+
+            default:
+              // range of 0 or 1, do nothing
+              break;
             }
 
-          //          g_assert(xsorted[strt] <= xsorted[pivot]);
-          //          g_assert(xsorted[pivot] <= xsorted[stop]);
-
+          // Pop the stack
           top -= 2;
-        }
-      else if (diff == 2)
-        {
-          --stop;
-          if (xsorted[strt] > xsorted[stop])
-            {
-              psort_swap(&xsorted[strt], &xsorted[stop]);
-              psort_swap(&ysorted[strt], &ysorted[stop]);
-            }
-          top -= 2;
-        }
-      else 
-        {
-          // range of 0 or 1 - just pop (well, shouldn't have bothered to push...)
-          top -= 2;
-        }
-    }
-  
-  delete [] stck;
+        } // else diff is small
+    } // while (top != 0) // i.e. while there's stuff on the stack
 
 #ifdef GNOME_ENABLE_DEBUG
   //  check_time(&timer);
@@ -902,6 +944,21 @@ Axis::generate_ticks(int nticks, double length)
 
       ++i;
     }
+
+  // Set defaults for all non-manual ticks
+  if (default_tick_font_ != 0)
+    {
+      vector<Tick>::iterator j = ticks_.begin();
+      while (j != ticks_.end())
+        {
+          if (!j->manual())
+            {
+              j->set_font(default_tick_font_);
+            }
+          
+          ++j;
+        }
+    }
 }
 
 
@@ -939,17 +996,85 @@ Rectangle::move_edges(double inc)
 
 ///////////////////// Tick
 
+Tick::Tick()
+  : pos_(0.0), len_(0.0), font_(0),
+    in_(false), out_(true), ruled_(false), manual_(false)
+{
+  
+}
+
+Tick::~Tick()
+{
+  if (font_ != 0)
+    if (font_->unref() == 0)
+      delete font_;
+}
+
 Tick::Tick(const Tick& src)
   : pos_(src.pos_), len_(src.len_),
     color_(src.color_), label_(src.label_),
+    font_(src.font_),
     in_(src.in_), out_(src.out_), 
     ruled_(src.ruled_), manual_(src.manual_)
-{
-  // FIXME ticks will eventually have a font, that's why this copy constructor 
-  //  is needed.
-
+{  
+  if (font_ != 0)
+    font_->ref();
 }
 
+Tick& 
+Tick::operator=(const Tick& src)
+{
+  if (&src == this)
+    return *this;
+
+  pos_ = src.pos_;
+  len_ = src.len_;
+  color_ = src.color_;
+  label_ = src.label_;
+  in_ = src.in_;
+  out_ = src.out_;
+  ruled_ = src.ruled_;
+  manual_ = src.manual_;
+
+  if (font_ != 0)
+    if (font_->unref() == 0)
+      delete font_;
+  
+  font_ = src.font_;
+  
+  if (font_ != 0)
+    font_->ref();
+
+  return *this;
+}
+
+void 
+Tick::set_font(VFont* vfont)
+{
+  if (vfont == font_)
+    return;
+
+  if (vfont != 0)
+    {
+      vfont->ref();
+    }
+
+  if (font_ != 0)
+    {
+      if (font_->unref() == 0)
+        delete font_;
+      
+      font_ = 0;
+    }
+
+  font_ = vfont;
+}
+
+VFont* 
+Tick::font() const
+{
+  return font_;
+}
 
 ////////////////// Axis
 
@@ -960,11 +1085,54 @@ Axis::Axis(const Axis& src)
     line_color_(src.line_color_),
     pos_(src.pos_),
     label_font_(src.label_font_),
-    line_(src.line_)
+    default_tick_font_(src.default_tick_font_),
+    line_(src.line_), 
+    hidden_(src.hidden_), label_hidden_(src.label_hidden_)
 {
   if (label_font_ != 0)
     label_font_->ref();
 
+  if (default_tick_font_ != 0)
+    default_tick_font_->ref();
+}
+
+Axis& 
+Axis::operator=(const Axis& src)
+{
+  if (&src == this)
+    return *this;
+
+  start_ = src.start_;
+  stop_ = src.stop_;
+  ticks_ = src.ticks_;
+  label_ = src.label_;
+  label_color_ = src.label_color_;
+  line_color_ = src.line_color_;
+  pos_ = src.pos_;
+  line_ = src.line_;
+  hidden_ = src.hidden_;
+  label_hidden_ = src.label_hidden_;
+
+  if (label_font_ != 0)
+    if (label_font_->unref() == 0)
+      delete label_font_;
+
+  label_font_ = src.label_font_;
+  
+  if (label_font_ != 0)
+    label_font_->ref();
+
+  
+  if (default_tick_font_ != 0)
+    if (default_tick_font_->unref() == 0)
+      delete default_tick_font_;
+
+  default_tick_font_ = src.default_tick_font_;
+  
+  if (default_tick_font_ != 0)
+    default_tick_font_->ref();  
+
+  return *this;
 }
 
 Axis::~Axis()
@@ -974,6 +1142,12 @@ Axis::~Axis()
       if (label_font_->unref() == 0)
         delete label_font_;
     }
+
+  if (default_tick_font_ != 0)
+    {
+      if (default_tick_font_->unref() == 0)
+        delete default_tick_font_;
+    }
 }
 
 VFont* 
@@ -988,6 +1162,11 @@ Axis::set_label_font(VFont* vfont)
   if (vfont == label_font_)
     return;
 
+  if (vfont != 0)
+    {
+      vfont->ref();
+    }
+
   if (label_font_ != 0)
     {
       if (label_font_->unref() == 0)
@@ -997,11 +1176,33 @@ Axis::set_label_font(VFont* vfont)
     }
 
   label_font_ = vfont;
-
-  if (label_font_ != 0)
-    {
-      label_font_->ref();
-    }
 }
 
+VFont* 
+Axis::default_tick_font() const
+{
+  return default_tick_font_;
+}
+
+void 
+Axis::set_default_tick_font(VFont* vfont)
+{
+  if (vfont == default_tick_font_)
+    return;
+
+  if (vfont != 0)
+    {
+      vfont->ref();
+    }
+
+  if (default_tick_font_ != 0)
+    {
+      if (default_tick_font_->unref() == 0)
+        delete default_tick_font_;
+      
+      default_tick_font_ = 0;
+    }
+
+  default_tick_font_ = vfont;
+}
 
