@@ -47,10 +47,12 @@ ScalarData::ScalarData(RealSet* rs)
 ScalarData::~ScalarData()
 {
 #ifdef GNOME_ENABLE_DEBUG
-  if (checked_out_) g_warning("ScalarData deleted with outstanding checkout!");
+  if (checked_out_) 
+    g_warning("ScalarData deleted with outstanding checkout!");
 #endif
 
-  if (set_ != 0) delete set_;
+  if (set_ != 0) 
+    delete set_;
 }
 
 gsize 
@@ -78,9 +80,9 @@ ScalarData::max() const
 }
 
 const double* 
-ScalarData::sorted_data()
+ScalarData::sorted_scalars()
 {
-  return set_->sorted_data(); // for now only. see note in .h
+  return set_->sorted_data();
 }
 
 void
