@@ -1891,6 +1891,20 @@ protected:
   }
 };
 
+class AddPrice : public AddLayer {
+public:
+  AddPrice()
+    : AddLayer("guppi-xy-add-price-layer", 
+               _("Add Price Bars Layer"),
+               _("Add price bars to the XY plot"))
+    {}
+protected:
+  virtual XyLayer* make_layer(XyPlot* plot) {
+    return new XyPriceBars(plot->state()->store());
+  }
+};
+
+
 ///
 
 class XyProperties : public MenuEntry {
@@ -1901,9 +1915,9 @@ public:
 };
 
 XyProperties::XyProperties() 
-  : MenuEntry("guppi-xy-show-properties", 
-              _("Properties"),
-              _("Properties dialog for the xy plot"))
+  : MenuEntry("guppi-xy-show-layer-properties", 
+              _("Layer Properties"),
+              _("Layer properties dialog for the XY plot"))
 {
 
 }
@@ -1918,6 +1932,31 @@ XyProperties::invoke(Plot* p)
 
 ///
 
+class XyAxisProperties : public MenuEntry {
+public:
+  XyAxisProperties();
+  
+  void invoke(Plot* p);
+};
+
+XyAxisProperties::XyAxisProperties() 
+  : MenuEntry("guppi-xy-show-axis-properties", 
+              _("Axis Properties"),
+              _("Axis properties dialog for the XY plot"))
+{
+
+}
+
+void
+XyAxisProperties::invoke(Plot* p)
+{
+  XyPlot* sp = static_cast<XyPlot*>(p);
+  
+  sp->axis_edit();
+}
+
+///
+
 #include "printer-gnome.h"
 #include "state-print.h"
 
@@ -1981,10 +2020,12 @@ guppi_gnome_init_xy()
 
   ps->add_menuentry(XyPlot::ID, new XyPrint);
   ps->add_menuentry(XyPlot::ID, new XyProperties);
+  ps->add_menuentry(XyPlot::ID, new XyAxisProperties);
   ps->add_menuentry(XyPlot::ID, new PaletteEdit);
   ps->add_menuentry(XyPlot::ID, new OptimalAxes);
   ps->add_menuentry(XyPlot::ID, new AddScatter);
   ps->add_menuentry(XyPlot::ID, new AddLine);
+  ps->add_menuentry(XyPlot::ID, new AddPrice);
 }
 
 
