@@ -250,6 +250,13 @@ XyDataLayer::x_data()
   return pair_->x_data();
 }
 
+const ScalarData* 
+XyDataLayer::x_data() const
+{
+  return pair_->x_data();
+}
+
+
 void 
 XyDataLayer::set_y_data(Data* d)
 {
@@ -271,6 +278,12 @@ XyDataLayer::y_data()
   return pair_->y_data();
 }
 
+const ScalarData* 
+XyDataLayer::y_data() const
+{
+  return pair_->y_data();
+}
+
 guint 
 XyDataLayer::npoints() const
 {
@@ -735,6 +748,16 @@ XyPriceBars::price_data(PriceType which)
 
   return price_data_[which];
 }
+
+
+const ScalarData* 
+XyPriceBars::price_data(PriceType which) const
+{
+  g_return_val_if_fail(which < PriceTypeEnd, 0);  
+
+  return price_data_[which];
+}
+
   
 void 
 XyPriceBars::set_time_data(Data* d)
@@ -761,6 +784,12 @@ XyPriceBars::time_data()
   return time_data_;
 }
 
+const Data* 
+XyPriceBars::time_data() const
+{
+  return time_data_;
+}
+
 guint 
 XyPriceBars::nbars() const
 {
@@ -845,7 +874,7 @@ XyPriceBars::change_data_values(DataStore* ds,
   if (!our_data(d))
     return;
 
-  pricebars_model.data_values_changed(this, d);
+  price_model.data_values_changed(this, d);
 }
 
 void 
@@ -857,7 +886,7 @@ XyPriceBars::change_data_values(DataStore* ds, Data* d)
   // Number of points in d may have changed; check this if we ever cache
   //  nbars
 
-  pricebars_model.data_values_changed(this, d);
+  price_model.data_values_changed(this, d);
 }
 
 void 
