@@ -28,6 +28,7 @@
 #include "canvas-scatter.h"
 #include "canvas-axis.h"
 #include "canvas-lineplot.h"
+#include "canvas-pricebars.h"
 #include "plotutils.h"
 #include "xyplotstate.h"
 #include "scalardata.h"
@@ -565,6 +566,198 @@ XyLineView::info_func(struct _GuppiLineplot* gs,
   *xy = sv->layer_; 
 }
 
+//////////////////////////////
+
+
+class XyPriceView : public XyLayerView,
+                    public XyPriceBars::View
+{
+public:
+  XyPriceView(XyPlotState* state, XyPriceBars* layer);
+  virtual ~XyPriceView();
+
+  virtual void realize(GnomeCanvasGroup* group);
+  virtual void unrealize();
+
+  virtual void update();
+
+  // XyLayer::View
+
+  virtual void change_x_axis(XyLayer* layer, 
+                             PlotUtil::PositionType old_pos, 
+                             PlotUtil::PositionType new_pos);
+
+  virtual void change_y_axis(XyLayer* layer, 
+                             PlotUtil::PositionType old_pos, 
+                             PlotUtil::PositionType new_pos);
+
+  virtual void change_hidden(XyLayer* layer,
+                             bool hidden);
+
+  // XyPriceBars::View
+
+  virtual void change_price_data(XyPriceBars* layer, 
+                                 XyPriceBars::PriceType which,
+                                 Data* old_data, Data* new_data);
+
+  virtual void change_time_data(XyPriceBars* layer, 
+                                Data* old_data, Data* new_data);
+  
+  virtual void change_data_values(XyPriceBars* layer, Data* data);
+
+  // ::View
+  virtual void destroy_model();
+
+  static void info_func(struct _GuppiPricebars* gs, 
+                        gpointer user_data, 
+                        XyPlotState** state,
+                        XyPriceBars** xy);
+
+private:
+  XyPriceBars* layer_;
+
+};
+
+XyPriceView::XyPriceView(XyPlotState* state, XyPriceBars* layer)
+  : XyLayerView(state), layer_(layer)
+{
+  g_return_if_fail(layer_ != 0);
+
+  layer_->ref();
+
+  layer_->layer_model.add_view(*this);
+  layer_->price_model.add_view(*this);
+}
+
+XyPriceView::~XyPriceView()
+{
+  if (item_ != 0)
+    unrealize();
+
+  layer_->layer_model.remove_view(*this);
+  layer_->price_model.remove_view(*this);
+
+  if (layer_->unref() == 0)
+    {
+      delete layer_;
+    }
+}
+
+void
+XyPriceView::realize(GnomeCanvasGroup* group)
+{
+  g_return_if_fail(item_ == 0);
+
+  item_ = gnome_canvas_item_new(GNOME_CANVAS_GROUP(group),
+                                guppi_pricebars_get_type(),
+                                NULL);
+
+  guppi_pricebars_set_info_func(GUPPI_PRICEBARS(item_), info_func, this);
+  
+  // We are going to ref our objects, so that the order of destruction
+  //  (us or canvas first) isn't important
+  gtk_object_ref(GTK_OBJECT(item_));
+
+  if (layer_->hidden())
+    gnome_canvas_item_hide(item_);
+}
+
+void
+XyPriceView::unrealize()
+{
+  if (item_ != 0) 
+    gtk_object_destroy(GTK_OBJECT(item_));  
+}
+
+void 
+XyPriceView::update()
+{
+  if (item_ != 0)
+    gnome_canvas_item_request_update(item_);
+}
+
+/// XyLayer::View
+
+void
+XyPriceView::change_x_axis(XyLayer* layer, 
+                             PlotUtil::PositionType old_pos, 
+                             PlotUtil::PositionType new_pos)
+{
+  if (item_ != 0)
+    gnome_canvas_item_request_update(item_);
+}
+
+void
+XyPriceView::change_y_axis(XyLayer* layer, 
+                             PlotUtil::PositionType old_pos, 
+                             PlotUtil::PositionType new_pos)
+{
+  if (item_ != 0)
+    gnome_canvas_item_request_update(item_);
+}
+
+void 
+XyPriceView::change_hidden(XyLayer* layer,
+                             bool hidden)
+{
+  if (item_ != 0)
+    {
+      if (hidden)
+        gnome_canvas_item_hide(item_);
+      else
+        gnome_canvas_item_show(item_);
+    }
+}
+
+/// XyPriceBars::View
+
+
+void 
+XyPriceView::change_price_data(XyPriceBars* layer, 
+                               XyPriceBars::PriceType which,
+                               Data* old_data, Data* new_data)
+{
+  if (item_ != 0)
+    gnome_canvas_item_request_update(item_);
+}
+
+void 
+XyPriceView::change_time_data(XyPriceBars* layer, 
+                              Data* old_data, Data* new_data)
+{
+  if (item_ != 0)
+    gnome_canvas_item_request_update(item_);
+}
+  
+void
+XyPriceView::change_data_values(XyPriceBars* layer, Data* data)
+{
+  if (item_ != 0)
+    gnome_canvas_item_request_update(item_);
+}
+
+/// ::View
+void
+XyPriceView::destroy_model()
+{
+#ifdef GNOME_ENABLE_DEBUG
+  g_warning("XyPriceBars destroyed from underneath XyPriceView");
+#endif
+}
+
+
+void 
+XyPriceView::info_func(struct _GuppiPricebars* gs, 
+                      gpointer user_data, 
+                      XyPlotState** state,
+                      XyPriceBars** xy)
+{
+  XyPriceView* sv = static_cast<XyPriceView*>(user_data);
+  
+  *state = sv->state_;
+  *xy = sv->layer_; 
+}
+
 ////////////////////////////////////////////////
 
 const char* XyPlot::ID = "guppi-xy";
@@ -572,7 +765,8 @@ const char* XyPlot::ID = "guppi-xy";
 XyPlot::XyPlot(DataStore* datastore)
   : Plot(ID, datastore), state_(0), 
     group_(0), layer_group_(0), bg_(0), 
-    xpos_(0.0), ypos_(0.0)
+    xpos_(0.0), ypos_(0.0),
+    axis_edit_(0)
 {
   memset(axes_,0,sizeof(GnomeCanvasItem*)*PlotUtil::PositionTypeEnd);
   memset(axis_labels_,0,sizeof(GnomeCanvasItem*)*PlotUtil::PositionTypeEnd);
@@ -705,10 +899,10 @@ XyPlot::realize(GnomeCanvasGroup* group)
     
     gnome_canvas_item_affine_relative (axis_labels_[PlotUtil::WEST], affine.raw());
 
-    affine.set_identity();
-    affine.rotate(90);    
+    Affine affine2;
+    affine2.rotate(90);    
 
-    gnome_canvas_item_affine_relative (axis_labels_[PlotUtil::EAST], affine.raw());
+    gnome_canvas_item_affine_relative (axis_labels_[PlotUtil::EAST], affine2.raw());
   }
 
   /// Realize each layer; we need to realize in the order they are in in the plot state,
@@ -863,6 +1057,19 @@ XyPlot::size_request(double* width, double* height)
   state_->size_request(width, height);
 }
 
+void
+XyPlot::axis_edit()
+{
+  // Demand-create, just to save some RAM and time
+  if (axis_edit_ == 0)
+    {
+      axis_edit_ = new XyAxisEdit;
+      axis_edit_->set_state(state_);
+    }
+
+  axis_edit_->edit();
+}
+
 const string& 
 XyPlot::name() const
 {
@@ -904,7 +1111,7 @@ XyPlot::change_axis(XyPlotState* state,
 
   if (axis_labels_[pos] != 0)
     {
-      if (axis.hidden())
+      if (axis.hidden() || axis.label_hidden() || !axis.labelled())
         {
           gnome_canvas_item_hide(axis_labels_[pos]);
         }
@@ -1042,6 +1249,9 @@ XyPlot::release_state()
     {
       edit_.set_state(0);
 
+      if (axis_edit_ != 0)
+        axis_edit_->set_state(0);
+
       // Destroy all the layer views
       map<XyLayer*,XyLayerView*>::iterator i = layer_views_.begin();
       
@@ -1078,6 +1288,8 @@ XyPlot::set_state(XyPlotState* state)
   state_->state_model.add_view(*this);
 
   edit_.set_state(state_);
+  if (axis_edit_ != 0)
+    axis_edit_->set_state(state_);
 
   // Create views for each layer
 
@@ -1159,6 +1371,11 @@ XyPlot::create_layer_view(XyLayer* layer)
                             (XyLine*)layer->cast_to_type(XyLayer::LineLayer));
       break;
 
+    case XyLayer::PriceLayer:
+      view = new XyPriceView(state_,
+                             (XyPriceBars*)layer->cast_to_type(XyLayer::PriceLayer));
+      break;
+
     case XyLayer::RelationLayer:
     default:
       view = 0;
