@@ -727,6 +727,52 @@ static bool get_min_max(XyLayer* layer, PlotUtil::PositionType pos,
             }
         }
     }
+  else if (tr->is_a(type_id, XyLayer::PriceLayer))
+    {
+      XyPriceBars* pl = 
+        (XyPriceBars*) layer->cast_to_type(XyLayer::PriceLayer);
+
+      g_assert(pl != 0);
+
+      if (pos == layer->x_axis())
+        {
+          Data* d = pl->time_data();
+          if (d != 0)
+            {
+              if (d->type() == Data::Scalar)
+                {
+                  ScalarData* sd = d->cast_to_scalar();
+
+                  *min = sd->min();
+                  *max = sd->max();
+                  retval = true;
+                }
+              // FIXME eventually the type can be date data.
+            }
+          else
+            {
+              // If there's no time data, just use sequential numbers.
+              guint nbars = pl->nbars();
+              if (nbars > 0)
+                {
+                  *min = 0;
+                  *max = nbars;
+                  retval = true;
+                }
+            }
+        }
+      else if (pos == layer->y_axis())
+        {
+          ScalarData* hi_sd = pl->price_data(XyPriceBars::Hi);
+          ScalarData* lo_sd = pl->price_data(XyPriceBars::Lo);
+          if (hi_sd && lo_sd)
+            {
+              *min = lo_sd->min();
+              *max = hi_sd->max();
+              retval = true;
+            }
+        }
+    }
 
   // eventually we might have a way to get min/max from relation layers
 
@@ -930,7 +976,7 @@ get_label_size(const Axis& axis, double* size)
 {
   static double default_label_size = 20.0;
 
-  if (axis.hidden())
+  if (axis.label_hidden() || axis.hidden() || !axis.labelled())
     {
       *size = 0.0;
       return;
